@@ -22,6 +22,7 @@ from vstools import (
 
 from .bm3d import BM3D as BM3DM
 from .bm3d import BM3DCPU, AbstractBM3D, BM3DCuda, BM3DCudaRTC, Profile
+from .dfttest import DFTTest
 from .knlm import DEVICETYPE, DeviceType, nl_means
 
 __all__ = [
@@ -90,9 +91,7 @@ class PrefilterBase(CustomIntEnum, metaclass=PrefilterMeta):
             pref_type = Prefilter.DFTTEST
 
         if pref_type == Prefilter.DFTTEST:
-            dftt_args = dict[str, Any](tbsize=1, slocation=[0.0, 4.0, 0.2, 9.0, 1.0, 15.0]) | kwargs
-
-            dfft = clip.dfttest.DFTTest(**dftt_args)
+            dftt = DFTTest(sloc={0.0: 4, 0.2: 9, 1.0: 15}, tr=0).denoise(clip, **kwargs)
 
             i, j = (scale_value(x, 8, bits, range_out=ColorRange.FULL) for x in (16, 75))
 
@@ -101,7 +100,7 @@ class PrefilterBase(CustomIntEnum, metaclass=PrefilterMeta):
                 f'x {i} < {peak} x {j} > 0 {peak} x {i} - {peak} {j} {i} - / * - ? ?'
             )
 
-            return dfft.std.MaskedMerge(clip, pref_mask, planes)
+            return dftt.std.MaskedMerge(clip, pref_mask, planes)
 
         if pref_type == Prefilter.NLMEANS:
             kwargs |= dict(strength=7.0, tr=1, sr=2, simr=2) | kwargs | dict(planes=planes)
@@ -131,9 +130,7 @@ class PrefilterBase(CustomIntEnum, metaclass=PrefilterMeta):
             else:
                 raise ValueError
 
-            sigma = kwargs.pop('sigma', sigma)
-
-            sigmas = [sigma if 0 in planes else 0, sigma if (1 in planes or 2 in planes) else 0]
+            sigmas = kwargs.pop('sigma', [sigma if 0 in planes else 0, sigma if (1 in planes or 2 in planes) else 0])
 
             bm3d_args = dict[str, Any](sigma=sigmas, radius=1, profile=profile) | kwargs
 
@@ -150,6 +147,12 @@ class PrefilterBase(CustomIntEnum, metaclass=PrefilterMeta):
 
             return upscaler.scale(boxblur, clip.width, clip.height)
 
+        if pref_type == Prefilter.GAUSS:
+            if 'sharp' not in kwargs and 'sigma' not in kwargs:
+                kwargs |= dict(sigma=1.5)
+
+            return gauss_blur(clip, **(kwargs | dict[str, Any](planes=planes)))
+
         if pref_type == Prefilter.GAUSSBLUR:
             if 'sharp' not in kwargs and 'sigma' not in kwargs:
                 kwargs |= dict(sigma=1.0)
@@ -164,7 +167,7 @@ class PrefilterBase(CustomIntEnum, metaclass=PrefilterMeta):
             if 'sharp' not in kwargs and 'sigma' not in kwargs:
                 kwargs |= dict(sigma=1.75)
 
-            strg = clamp(kwargs.pop('strenght', 50 if pref_type == Prefilter.GAUSSBLUR2 else 90), 0, 98) + 1
+            strg = clamp(kwargs.pop('strength', 50 if pref_type == Prefilter.GAUSSBLUR2 else 90), 0, 98) + 1
 
             gaussblur = gauss_blur(boxblur, **(kwargs | dict[str, Any](planes=planes)))
 
@@ -203,7 +206,17 @@ class PrefilterBase(CustomIntEnum, metaclass=PrefilterMeta):
 
             return bilateral(clip, baseS, baseR, ref, **kwargs)
 
-        return clip
+        if pref_type is Prefilter.BMLATERAL:
+            sigma = kwargs.pop('sigma', 1.5)
+            tr = kwargs.pop('tr', 2)
+            radius = kwargs.pop('radius', 7)
+            cuda = kwargs.pop('gpu', hasattr(core, 'bm3dcuda'))
+
+            den = Prefilter.BM3D(
+                clip, planes, sigma=[10.0, 8.0] if cuda else [8.0, 6.4], radius=tr, gpu=cuda, **kwargs
+            )
+
+            return Prefilter.BILATERAL(den, planes, sigmaS=[sigma, sigma / 3], sigmaR=radius / 255)
 
 
 class Prefilter(PrefilterBase):
@@ -249,6 +262,9 @@ class Prefilter(PrefilterBase):
     SCALEDBLUR = 7
     """Perform blurring at a scaled-down resolution, then scale it back up."""
 
+    GAUSS = 13
+    """Simply Gaussian blur."""
+
     GAUSSBLUR = 8
     """Gaussian blurred, then postprocessed to remove low frequencies."""
 
@@ -261,6 +277,9 @@ class Prefilter(PrefilterBase):
     BILATERAL = 11
     """Classic bilateral filtering or edge-preserving bilateral multi pass filtering."""
 
+    BMLATERAL = 14
+    """BM3D + BILATERAL blurring."""
+
     if TYPE_CHECKING:
         from .prefilters import Prefilter
 
@@ -463,6 +482,25 @@ class Prefilter(PrefilterBase):
             :return:            Preprocessed clip.
             """
 
+        @overload
+        def __call__(  # type: ignore
+            self: Literal[Prefilter.BMLATERAL], clip: vs.VideoNode, /, planes: PlanesT = None,
+            sigma: float = 1.5, radius: float = 7, tr: int = 2, gpu: bool | None = None, **kwargs: Any
+        ) -> vs.VideoNode:
+            """
+            BM3D + BILATERAL smoothing.
+
+            :param clip:        Clip to be preprocessed.
+            :param planes:      Planes to be preprocessed.
+            :param sigma:       ``Bilateral`` spatial weight sigma.
+            :param radius:      ``Bilateral`` radius weight sigma.
+            :param tr:          Temporal radius for BM3D.
+            :param gpu:         Whether to process with GPU or not. None is auto.
+            :param **kwargs:    Kwargs passed to BM3D.
+
+            :return:            Partial Prefilter.
+            """
+
         @overload
         def __call__(self, clip: vs.VideoNode, /, planes: PlanesT = None, **kwargs: Any) -> vs.VideoNode:
             """
@@ -664,6 +702,24 @@ class Prefilter(PrefilterBase):
             :return:            Partial Prefilter.
             """
 
+        @overload
+        def __call__(  # type: ignore
+            self: Literal[Prefilter.BMLATERAL], *, planes: PlanesT = None,
+            sigma: float = 1.5, radius: float = 7, tr: int = 2, gpu: bool | None = None, **kwargs: Any
+        ) -> vs.VideoNode:
+            """
+            BM3D + BILATERAL smoothing.
+
+            :param planes:      Planes to be preprocessed.
+            :param sigma:       ``Bilateral`` spatial weight sigma.
+            :param radius:      ``Bilateral`` radius weight sigma.
+            :param tr:          Temporal radius for BM3D.
+            :param gpu:         Whether to process with GPU or not. None is auto.
+            :param **kwargs:    Kwargs passed to BM3D.
+
+            :return:            Partial Prefilter.
+            """
+
         @overload
         def __call__(self, *, planes: PlanesT = None, **kwargs: Any) -> PrefilterPartial:
             """
